@@ -59,7 +59,6 @@ tracker_data_search_text (TrackerDBInterface *iface,
 	gint		     count;
 	const gchar	    *procedure;
 	GArray		    *services = NULL;
-	GSList		    *duds = NULL;
 	guint		     i = 0;
 
 	g_return_val_if_fail (TRACKER_IS_DB_INTERFACE (iface), NULL);
@@ -159,9 +158,6 @@ tracker_data_search_text (TrackerDBInterface *iface,
 
 			g_free (path);
 			g_object_unref (result_set);
-		} else {
-			g_message ("Dud hit for search detected");
-			duds = g_slist_prepend (duds, &rank);
 		}
 	}
 
@@ -169,29 +165,6 @@ tracker_data_search_text (TrackerDBInterface *iface,
 		tracker_db_interface_end_transaction (iface);
 	}
 
-	/* Delete duds */
-	if (duds) {
-		TrackerDBIndex *file_index;
-		TrackerDBIndex *email_index;
-		GSList	       *words, *w;
-
-		words = tracker_query_tree_get_words (tree);
-		file_index = tracker_db_index_manager_get_index (TRACKER_DB_INDEX_FILE);
-		email_index = tracker_db_index_manager_get_index (TRACKER_DB_INDEX_EMAIL);
-
-		for (w = words; w; w = w->next) {
-			tracker_db_index_remove_dud_hits (file_index,
-							  (const gchar *) w->data,
-							  duds);
-			tracker_db_index_remove_dud_hits (email_index,
-							  (const gchar *) w->data,
-							  duds);
-		}
-
-		g_slist_free (words);
-		g_slist_free (duds);
-	}
-
 	g_object_unref (tree);
 	g_array_free (hits, TRUE);
 
@@ -509,9 +482,9 @@ tracker_data_search_files_get (TrackerDBInterface *iface,
 	g_return_val_if_fail (folder_path != NULL, NULL);
 
 	result_set = tracker_data_manager_exec_proc (iface,
-					   "SelectFileChild",
-					   folder_path,
-					   NULL);
+						     "GetFileChildren",
+						     folder_path,
+						     NULL);
 	array = g_ptr_array_new ();
 
 	if (result_set) {
@@ -664,6 +637,14 @@ is_data_type_numeric (TrackerFieldType type)
 		type == TRACKER_FIELD_TYPE_DOUBLE;
 }
 
+static gboolean
+is_data_type_text (TrackerFieldType type) 
+{
+	return 
+		type == TRACKER_FIELD_TYPE_STRING ||
+		type == TRACKER_FIELD_TYPE_INDEX;
+}
+
 static TrackerFieldData *
 tracker_metadata_add_metadata_field (TrackerDBInterface *iface,
 				     const gchar        *service,
@@ -1195,6 +1176,241 @@ tracker_data_search_get_unique_values_with_count_and_sum (const gchar	      *ser
 	return result_set;
 }
 
+TrackerDBResultSet *
+tracker_data_search_get_unique_values_with_concat_count_and_sum (const gchar	      *service_type,
+								 gchar		     **fields,
+								 const gchar	      *query_condition,
+								 const gchar          *concat_field,
+								 const gchar	      *count_field,
+								 const gchar          *sum_field,
+								 gboolean	       order_desc,
+								 gint		       offset,
+								 gint		       max_hits,
+								 GError	             **error)
+{
+	TrackerDBInterface *iface;
+	TrackerDBResultSet *result_set = NULL;
+
+	GSList		   *field_list = NULL;
+	gchar		   *str_offset, *str_limit;
+
+	GString		   *sql_select;
+	GString		   *sql_from;
+	GString		   *sql_where;
+	GString		   *sql_order;
+	GString		   *sql_group;
+	gchar		   *sql;
+
+	gchar		   *rdf_where;
+	gchar		   *rdf_from;
+	GError		   *actual_error = NULL;
+
+	guint		    i;
+
+	g_return_val_if_fail (service_type != NULL, NULL);
+	g_return_val_if_fail (fields != NULL, NULL);
+	g_return_val_if_fail (query_condition != NULL, NULL);
+
+	if (!tracker_ontology_service_is_valid (service_type)) {
+		g_set_error (error, TRACKER_DBUS_ERROR, 0,
+			     "Service_Type '%s' is invalid or has not been implemented yet",
+			     service_type);
+		return NULL;
+	}
+
+	iface = tracker_db_manager_get_db_interface_by_service (service_type);
+
+	sql_select = g_string_new ("SELECT DISTINCT ");
+	sql_from   = g_string_new ("\nFROM Services AS S ");
+	sql_where  = g_string_new ("\nWHERE ");
+	sql_order  = g_string_new ("\nORDER BY ");
+	sql_group  = g_string_new ("\nGROUP BY ");
+
+
+	for (i = 0; i < g_strv_length (fields); i++) {
+		TrackerFieldData *fd;
+
+		fd = tracker_metadata_add_metadata_field (iface, service_type, &field_list, fields[i], TRUE, FALSE);
+
+		if (!fd) {
+			g_string_free (sql_select, TRUE);
+			g_string_free (sql_from, TRUE);
+			g_string_free (sql_where, TRUE);
+			g_string_free (sql_order, TRUE);
+			g_string_free (sql_group, TRUE);
+
+			g_set_error (error, TRACKER_DBUS_ERROR, 0,
+				     "Invalid or non-existant metadata type '%s' specified",
+				     fields[i]);
+			return NULL;
+		}
+
+		if (i) {
+			g_string_append_printf (sql_select, ",");
+			g_string_append_printf (sql_order, ",");
+			g_string_append_printf (sql_group, ",");
+		}
+
+		g_string_append_printf (sql_select, "COALESCE(%s,'')", tracker_field_data_get_select_field (fd));
+		g_string_append_printf (sql_order, " %s %s",
+					tracker_field_data_get_select_field (fd),
+					order_desc ? "DESC" : "ASC" );
+		g_string_append_printf (sql_group, "COALESCE(%s,'')", tracker_field_data_get_select_field (fd));
+
+	}
+
+	if (concat_field && !(tracker_is_empty_string (concat_field))) {
+		TrackerFieldData *fd;
+		TrackerFieldType  data_type;
+
+		fd = tracker_metadata_add_metadata_field (iface, service_type, &field_list, concat_field, TRUE, FALSE);
+
+		if (!fd) {
+			g_string_free (sql_select, TRUE);
+			g_string_free (sql_from, TRUE);
+			g_string_free (sql_where, TRUE);
+			g_string_free (sql_order, TRUE);
+			g_string_free (sql_group, TRUE);
+
+			g_set_error (error, TRACKER_DBUS_ERROR, 0,
+				     "Invalid or non-existant metadata type '%s' specified",
+				     sum_field);
+			return NULL;
+		}
+
+		data_type = tracker_field_data_get_data_type (fd);
+
+		if (!is_data_type_text (data_type)) {
+			g_string_free (sql_select, TRUE);
+			g_string_free (sql_from, TRUE);
+			g_string_free (sql_where, TRUE);
+			g_string_free (sql_order, TRUE);
+			g_string_free (sql_group, TRUE);
+
+			g_set_error (error, TRACKER_DBUS_ERROR, 0,
+				     "Cannot concatenate '%s': this metadata type is not text",
+				     sum_field);
+			return NULL;
+		}
+
+		g_string_append_printf (sql_select, ", GROUP_CONCAT (DISTINCT %s)", tracker_field_data_get_select_field (fd));
+	}
+
+	if (count_field && !(tracker_is_empty_string (count_field))) {
+		TrackerFieldData *fd;
+
+		if (strcmp (count_field, "*")) {
+			fd = tracker_metadata_add_metadata_field (iface, service_type, &field_list, count_field, TRUE, FALSE);
+			
+			if (!fd) {
+				g_string_free (sql_select, TRUE);
+				g_string_free (sql_from, TRUE);
+				g_string_free (sql_where, TRUE);
+				g_string_free (sql_order, TRUE);
+				g_string_free (sql_group, TRUE);
+				
+				g_set_error (error, TRACKER_DBUS_ERROR, 0,
+					     "Invalid or non-existant metadata type '%s' specified",
+					     count_field);
+				return NULL;
+			}
+			
+			g_string_append_printf (sql_select, ", COUNT (DISTINCT COALESCE(%s,''))", tracker_field_data_get_select_field (fd));
+		} else {
+				g_string_append_printf (sql_select, ", COUNT (DISTINCT S.ID)");		
+		}
+	}
+
+	if (sum_field && !(tracker_is_empty_string (sum_field))) {
+		TrackerFieldData *fd;
+		TrackerFieldType  data_type;
+
+		fd = tracker_metadata_add_metadata_field (iface, service_type, &field_list, sum_field, TRUE, FALSE);
+
+		if (!fd) {
+			g_string_free (sql_select, TRUE);
+			g_string_free (sql_from, TRUE);
+			g_string_free (sql_where, TRUE);
+			g_string_free (sql_order, TRUE);
+			g_string_free (sql_group, TRUE);
+
+			g_set_error (error, TRACKER_DBUS_ERROR, 0,
+				     "Invalid or non-existant metadata type '%s' specified",
+				     sum_field);
+			return NULL;
+		}
+
+		data_type = tracker_field_data_get_data_type (fd);
+
+		if (!is_data_type_numeric (data_type)) {
+			g_string_free (sql_select, TRUE);
+			g_string_free (sql_from, TRUE);
+			g_string_free (sql_where, TRUE);
+			g_string_free (sql_order, TRUE);
+			g_string_free (sql_group, TRUE);
+
+			g_set_error (error, TRACKER_DBUS_ERROR, 0,
+				     "Cannot sum '%s': this metadata type is not numeric",
+				     sum_field);
+			return NULL;
+		}
+
+		g_string_append_printf (sql_select, ", SUM (%s)", tracker_field_data_get_select_field (fd));
+	}
+
+	tracker_rdf_filter_to_sql (iface, query_condition, service_type,
+				   &field_list, &rdf_from, &rdf_where, &actual_error);
+
+	if (actual_error) {
+		g_string_free (sql_select, TRUE);
+		g_string_free (sql_from, TRUE);
+		g_string_free (sql_where, TRUE);
+		g_string_free (sql_order, TRUE);
+		g_string_free (sql_group, TRUE);
+
+		g_propagate_error (error, actual_error);
+
+		return NULL;
+	}
+
+	g_string_append_printf (sql_from, " %s ", rdf_from);
+	g_string_append_printf (sql_where, " %s ", rdf_where);
+
+	g_free (rdf_from);
+	g_free (rdf_where);
+
+	str_offset = tracker_gint_to_string (offset);
+	str_limit = tracker_gint_to_string (metadata_sanity_check_max_hits (max_hits));
+
+	g_string_append_printf (sql_order, " LIMIT %s,%s", str_offset, str_limit);
+
+	sql = g_strconcat (sql_select->str, " ",
+			   sql_from->str, " ",
+			   sql_where->str, " ",
+			   sql_group->str, " ",
+			   sql_order->str, NULL);
+
+	g_free (str_offset);
+	g_free (str_limit);
+
+	g_string_free (sql_select, TRUE);
+	g_string_free (sql_from, TRUE);
+	g_string_free (sql_where, TRUE);
+	g_string_free (sql_order, TRUE);
+	g_string_free (sql_group, TRUE);
+
+	g_slist_foreach (field_list, (GFunc) g_object_unref, NULL);
+	g_slist_free (field_list);
+
+	g_message ("Unique values query executed:\n%s", sql);
+
+	result_set =  tracker_db_interface_execute_query (iface, NULL, "%s", sql);
+
+	g_free (sql);
+
+	return result_set;
+}
+
 gint
 tracker_data_search_get_sum (const gchar	 *service_type,
 			     const gchar	 *field,
